import sys
import click

class NagErrors(object):
    '''

    The NagErrors object contains the processed Nagios Errors

    Args:
        - None -

    Attributes:
        critical (list of str's): The list of Critical Errors
        warning (list of str's): The list of Warning Errors
        unknown (list of str's): The list of Unknown Errors
    '''
    def __init__(self):
        self.critical = []
        self.warning = []
        self.unknown = []

    def addRecord(self, etype, etext):
        '''
        Add a record of a specific type

        Args:
            etype (enum str): One of 'crititcal', 'warning', 'unknown', the type of error
            etext (str): The descriptive text of the error
        '''
        if etype == 'critical':
            self.critical.append(etext)
        elif etype == 'warning':
            self.warning.append(etext)
        else:
            self.unknown.append(etext)

    def addCritical(self, etext):
        '''
        Add a critical record

        Args:
            etext (str): The descriptive text of the error
        '''
        self.addRecord('critical', etext)
    
    def addWarning(self, etext):
        '''
        Add a warning record

        Args:
            etext (str): The descriptive text of the error
        '''
        self.addRecord('warning', etext)
    
    def addUnknown(self, etext):
        '''
        Add a unknown record

        Args:
            etext (str): The descriptive text of the error
        '''
        self.addRecord('unknown', etext)

    def formatStatus(self, stype, sarr):
        '''
        Format an array of status lines

        Args:
            stype (str): Error Type of array
            sarr (List of str's): Error Messages

        Returns:
            str: Formated list of errors
        '''
        ret = ""
        if len(sarr) == 1:
            ret += "{} {}\r\n".format(stype, sarr[0])
        else:
            ret += "{}:\r\n".format(stype)
            for l in sarr:
                ret += "    {}\r\n".format(l)
        return(ret)

    def printStatus(self):
        '''
        Print properly formatted Nagios status information

        Args:
            - none -
        '''
        if len(self.unknown) > 0 :
            click.echo(self.formatStatus('UNKNOWN', self.unknown))
        if len(self.critical) > 0 :
            click.echo(self.formatStatus('CRITICAL', self.critical))
        if len(self.warning) > 0 :
            click.echo(self.formatStatus('WARNING', self.warning))
        if len(self.unknown) + len(self.critical) + len(self.warning) == 0 :
            click.echo('OK')
        
    def getExitCode(self):
        '''
        Get the returncode that should be exited with.

        Args:
            - none -

        Returns:
            int: Calculated returncode based on the current status lines
        '''
        if len(self.unknown) > 0 :
            return 3
        if len(self.critical) > 0 :
            return 2
        if len(self.warning) > 0 :
            return 1
        return 0

    def doExit(self):
        '''
        Exit with the proper returncode

        Args:
            - none -
        '''
        sys.exit(self.getExitCode())


class NagEval(object):
    '''

    The NagEval object provides common comparison/evaluators for testdata

    Args:
        errObj (NagErrors): NagErrors object used to store results

    Attributes:
        errObj (NagErrors): NagErrors object used to store results
    '''
    def __init__(self, errObj):
        self.errObj = errObj


    def evalListEnum(self, values, emptyStatus="UNKNOWN", unknownValueStatus="UNKNOWN", okValues=[], warningValues=[], criticalValues=[], unknownValues=[], prefixText="", postfixText= ""):
        '''
        Evaluate a list of values based on lists of enumerated values

        Args:
            values (List of str/num): Values to test
            emptyStatus (str): Result if values is an empty list
            unknownValueStatus (str): Status to assign to values that are not in any lists (OK, WARNING, CRITICAL, or UNKNOWN)
            okValues (List of str/num): Values to match for OK Status
            warningValues (List of str/num): Values to match for Warning Status
            criticalValues (List of str/num): Values to match for Critical Status
            unknownValues (List of str/num): Values to match for Unknown Status
            prefixText (str): String to prefix error reports
            postfixText (str): String to append to error reports

        Effects:
            self.errObj (NagErrors): Updated with error strings

        Returns:
            str: Result of test, One of (OK,WARNING,CRITICAL,UNKNOWN)
        '''
        if (type(values) == list and len(values) == 0) or (type(values) != list and values == ""):
            if emptyStatus != "OK":
                self.errObj.addRecord(emptyStatus.lower(),"{}list is Empty{}".format(prefixText, postfixText))
            return(emptyStatus)
        if type(values) != list:
            return self.evalEnum(values, unknownValueStatus, okValues, warningValues, criticalValues, unknownValues, prefixText, postfixText)

        ret = "OK"
        rvals = {"OK": 0, "WARNING": 1, "CRITICAL": 2, "UNKNOWN": 3}
        for value in values:
            r = self.evalEnum(value, unknownValueStatus, okValues, warningValues, criticalValues, unknownValues, prefixText, postfixText)
            if rvals[r] > rvals[ret]:
                ret = r
        return ret


    def evalEnum(self, value, defaultStatus="UNKNOWN", okValues=[], warningValues=[], criticalValues=[], unknownValues=[], prefixText="", postfixText= ""):
        '''
        Evaluate a value based on lists of enumerated values

        Args:
            value (str/num): Value to test
            defaultStatus (str): Result if value does not match any list
            okValues (List of str/num): Values to match for OK Status
            warningValues (List of str/num): Values to match for Warning Status
            criticalValues (List of str/num): Values to match for Critical Status
            unknownValues (List of str/num): Values to match for Unknown Status
            prefixText (str): String to prefix error reports
            postfixText (str): String to append to error reports

        Effects:
            self.errObj (NagErrors): Updated with error strings

        Returns:
            str: Result of test, One of (OK,WARNING,CRITICAL,UNKNOWN)
        '''
        if value in unknownValues:
            self.errObj.addUnknown("{}value is {}{}".format(prefixText, value, postfixText))
            return("UNKNOWN")
        if value in criticalValues:
            self.errObj.addCritical("{}value is {}{}".format(prefixText, value, postfixText))
            return("CRITICAL")
        if value in warningValues:
            self.errObj.addWarning("{}value is {}{}".format(prefixText, value, postfixText))
            return("WARNING")
        if value in okValues:
            return("OK")
        if defaultStatus != "OK":
            self.errObj.addRecord(defaultStatus.lower(), "{}value {} not found{}".format(prefixText, value, postfixText))
        return(defaultStatus)
    

    def evalListNumberAsc(self, values, emptyStatus="UNKNOWN", warningAbove=None, criticalAbove=None, prefixText="", postfixText="", numberUnits=""):
        '''
        Evaluate a list of values based on ascending thresholds

        Args:
            values (List of num): Values to test
            emptyStatus (str): Result if values is an empty list
            warningAbove (num): Generate a warning if value is above this threshold
            criticalAbove (num): Generate a critical error if value is above this threshold
            prefixText (str): String to prefix error reports
            postfixText (str): String to append to error reports
            numberUnits (str): Units to append to numbers in error reports

        Effects:
            self.errObj (NagErrors): Updated with error strings

        Returns:
            str: Result of test, One of (OK,WARNING,CRITICAL,UNKNOWN)
        '''
        if (type(values) == list and len(values) == 0) or (type(values) != list and values == ""):
            if emptyStatus != "OK":
                self.errObj.addRecord(emptyStatus.lower(),"{}list is Empty{}".format(prefixText, postfixText))
            return(emptyStatus)
        if type(values) != list:
            return self.evalNumberAsc(values, warningAbove, criticalAbove, prefixText, postfixText, numberUnits)

        ret = "OK"
        rvals = {"OK": 0, "WARNING": 1, "CRITICAL": 2, "UNKNOWN": 3}
        for value in values:
            r = self.evalNumberAsc(value, warningAbove, criticalAbove, prefixText, postfixText, numberUnits)
            if rvals[r] > rvals[ret]:
                ret = r
        return ret


    def evalNumberAsc(self, value, warningAbove=None, criticalAbove=None, prefixText="", postfixText="", numberUnits=""):
        '''
        Evaluate a number, generate warning or critical if above certain thresholds

        Args:
            value (num): Value to test
            warningAbove (num): Generate a warning if value is above this threshold
            criticalAbove (num): Generate a critical error if value is above this threshold
            prefixText (str): String to prefix error reports
            postfixText (str): String to append to error reports
            numberUnits (str): Units to append to numbers in error reports

        Effects:
            self.errObj (NagErrors): Updated with error strings

        Returns:
            str: Result of test, One of (OK,WARNING,CRITICAL)
        '''
        if criticalAbove is not None and value > criticalAbove:
            self.errObj.addCritical("{}{}{} is > {}{}{}".format(prefixText, value, numberUnits, criticalAbove, numberUnits, postfixText))
            return("CRITICAL")
        if warningAbove is not None and value > warningAbove:
            self.errObj.addWarning("{}{}{} is > {}{}{}".format(prefixText, value, numberUnits, warningAbove, numberUnits, postfixText))
            return("WARNING")
        return("OK")
    

    def evalListNumberDesc(self, values, emptyStatus="UNKNOWN", warningBelow=None, criticalBelow=None, prefixText="", postfixText="", numberUnits=""):
        '''
        Evaluate a list of values based on descending thresholds

        Args:
            values (List of num): Values to test
            emptyStatus (str): Result if values is an empty list
            warningBelow (num): Generate a warning if value is below this threshold
            criticalBelow (num): Generate a critical error if value is below this threshold
            prefixText (str): String to prefix error reports
            postfixText (str): String to append to error reports
            numberUnits (str): Units to append to numbers in error reports

        Effects:
            self.errObj (NagErrors): Updated with error strings

        Returns:
            str: Result of test, One of (OK,WARNING,CRITICAL,UNKNOWN)
        '''
        if (type(values) == list and len(values) == 0) or (type(values) != list and values == ""):
            if emptyStatus != "OK":
                self.errObj.addRecord(emptyStatus.lower(),"{}list is Empty{}".format(prefixText, postfixText))
            return(emptyStatus)
        if type(values) != list:
            return self.evalNumberDesc(values, warningBelow, criticalBelow, prefixText, postfixText, numberUnits)

        ret = "OK"
        rvals = {"OK": 0, "WARNING": 1, "CRITICAL": 2, "UNKNOWN": 3}
        for value in values:
            r = self.evalNumberDesc(value, warningBelow, criticalBelow, prefixText, postfixText, numberUnits)
            if rvals[r] > rvals[ret]:
                ret = r
        return ret
    
    def evalNumberDesc(self, value, warningBelow=None, criticalBelow=None, prefixText="", postfixText= "", numberUnits=""):
        '''
        Evaluate a number, generate warning or critical if below certain thresholds

        Args:
            value (num): Value to test
            warningBelow (num): Generate a warning if value is below this threshold
            criticalBelow (num): Generate a critical error if value is below this threshold
            prefixText (str): String to prefix error reports
            postfixText (str): String to append to error reports
            numberUnits (str): Units to append to numbers in error reports

        Effects:
            self.errObj (NagErrors): Updated with error strings

        Returns:
            str: Result of test, One of (OK,WARNING,CRITICAL)
        '''
        if criticalBelow is not None and value < criticalBelow:
            self.errObj.addCritical("{}{}{} is < {}{}{}".format(prefixText, value, numberUnits, criticalBelow, numberUnits, postfixText))
            return("CRITICAL")
        if warningBelow is not None and value < warningBelow:
            self.errObj.addWarning("{}{}{} is < {}{}{}".format(prefixText, value, numberUnits, warningBelow, numberUnits, postfixText))
            return("WARNING")
        return("OK")



from unittest import TestCase

from NagiosCheckHelper import NagErrors, NagEval

class TestNagEval_evalEnum(TestCase):

    def test_default(self):
        eo = NagErrors()
        ev = NagEval(eo)

        ev.evalEnum('Junk', defaultStatus='OK')
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)

        ev.evalEnum('Junk', defaultStatus='WARNING')
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 1)
        
        ev.evalEnum('Junk', defaultStatus='CRITICAL')
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)
        
        ev.evalEnum('Junk', defaultStatus='UNKNOWN')
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 1)
        self.assertEqual(eo.getExitCode(), 3)

    def test_bin_value(self):
        eo = NagErrors()
        ev = NagEval(eo)

        ev.evalEnum('OKVal', okValues=['OKVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)

        ev.evalEnum('WarningVal', warningValues=['WarningVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 1)
        
        ev.evalEnum('CriticalVal', criticalValues=['CriticalVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)
        
        ev.evalEnum('UnknownVal', unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 1)
        self.assertEqual(eo.getExitCode(), 3)

        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalEnum('OKVal', okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)

        ev.evalEnum('WarningVal', okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 1)
        
        ev.evalEnum('CriticalVal', okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)
        
        ev.evalEnum('UnknownVal', okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 1)
        self.assertEqual(eo.getExitCode(), 3)

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalEnum('Warning', warningValues=['Warning'], prefixText="Test ")
        self.assertEqual(eo.warning[0], "Test value is Warning")

    def test_postfixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalEnum('Warning', warningValues=['Warning'], postfixText=" String")
        self.assertEqual(eo.warning[0], "value is Warning String")


class TestNagEval_evalListEnum(TestCase):

    def test_default(self):
        eo = NagErrors()
        ev = NagEval(eo)

        ev.evalListEnum(['Junk'], unknownValueStatus='OK')
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)

        ev.evalListEnum(['Junk'], unknownValueStatus='WARNING')
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 1)
        
        ev.evalListEnum(['Junk'], unknownValueStatus='CRITICAL')
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)
        
        ev.evalListEnum(['Junk'], unknownValueStatus='UNKNOWN')
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 1)
        self.assertEqual(eo.getExitCode(), 3)

    def test_bin_value(self):
        eo = NagErrors()
        ev = NagEval(eo)

        ev.evalListEnum(['OKVal'], okValues=['OKVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)

        ev.evalListEnum(['WarningVal'], warningValues=['WarningVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 1)
        
        ev.evalListEnum(['CriticalVal'], criticalValues=['CriticalVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)
        
        ev.evalListEnum(['UnknownVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 1)
        self.assertEqual(eo.getExitCode(), 3)

        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListEnum(['OKVal'], okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)

        ev.evalListEnum(['WarningVal'], okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 1)
        
        ev.evalListEnum(['CriticalVal'], okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)
        
        ev.evalListEnum(['UnknownVal'], okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 1)
        self.assertEqual(eo.getExitCode(), 3)
    
    def test_bin_valuelist(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListEnum(['OKVal', 'OKVal', 'WarningVal', 'WarningVal', 'CriticalVal'], okValues=['OKVal'], warningValues=['WarningVal'], criticalValues=['CriticalVal'], unknownValues=['UnknownVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 2)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)
    
    def test_unknownValues(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListEnum(['junkVal'], unknownValueStatus="OK")
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)

        ev.evalListEnum(['junkVal'], unknownValueStatus="WARNING")
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 1)
        
        ev.evalListEnum(['junkVal'], unknownValueStatus="CRITICAL")
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)
        
        ev.evalListEnum(['junkVal'])
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 1)
        self.assertEqual(eo.getExitCode(), 3)

    def test_emptyList(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListEnum([], emptyStatus="OK")
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)
        ev.evalListEnum([], emptyStatus="WARNING")
        self.assertEqual(eo.warning[0], "list is Empty")
        self.assertEqual(eo.getExitCode(), 1)
    
    def test_string(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListEnum('WarningVal', warningValues=['WarningVal'])
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.warning[0], "value is WarningVal")
        self.assertEqual(eo.getExitCode(), 1)

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListEnum(['Warning'], warningValues=['Warning'], prefixText="Test ")
        self.assertEqual(eo.warning[0], "Test value is Warning")

    def test_postfixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListEnum(['Warning'], warningValues=['Warning'], postfixText=" String")
        self.assertEqual(eo.warning[0], "value is Warning String")


class TestNagEval_evalNumberAsc(TestCase):

    def test_low(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(20, warningAbove=40, criticalAbove=50)
        self.assertEqual(eo.getExitCode(), 0)
    
    def test_mid(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(45, warningAbove=40, criticalAbove=50)
        self.assertEqual(eo.getExitCode(), 1)

    def test_high(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(55, warningAbove=40, criticalAbove=50)
        self.assertEqual(eo.getExitCode(), 2)

    def test_negative_low(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(-20, warningAbove=-10, criticalAbove=-5)
        self.assertEqual(eo.getExitCode(), 0)
    
    def test_negative_mid(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(-8, warningAbove=-10, criticalAbove=-5)
        self.assertEqual(eo.getExitCode(), 1)
    
    def test_negative_high(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(-3, warningAbove=-10, criticalAbove=-5)
        self.assertEqual(eo.getExitCode(), 2)

    def test_swappedTresholds(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(51, warningAbove=50, criticalAbove=40)
        self.assertEqual(eo.getExitCode(), 2)

    def test_numberUnits(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(45, warningAbove=40, numberUnits="sec")
        self.assertEqual(eo.warning[0], "45sec is > 40sec")

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(45, warningAbove=40, prefixText="Test ")
        self.assertEqual(eo.warning[0], "Test 45 is > 40")

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberAsc(45, warningAbove=40, postfixText=" Units")
        self.assertEqual(eo.warning[0], "45 is > 40 Units")


class TestNagEval_evalListNumberAsc(TestCase):

    def test_low(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([20], warningAbove=40, criticalAbove=50)
        self.assertEqual(eo.getExitCode(), 0)
    
    def test_mid(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([45], warningAbove=40, criticalAbove=50)
        self.assertEqual(eo.getExitCode(), 1)

    def test_high(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([55], warningAbove=40, criticalAbove=50)
        self.assertEqual(eo.getExitCode(), 2)

    def test_negative_low(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([-20], warningAbove=-10, criticalAbove=-5)
        self.assertEqual(eo.getExitCode(), 0)
    
    def test_negative_mid(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([-8], warningAbove=-10, criticalAbove=-5)
        self.assertEqual(eo.getExitCode(), 1)
    
    def test_negative_high(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([-3], warningAbove=-10, criticalAbove=-5)
        self.assertEqual(eo.getExitCode(), 2)

    def test_swappedTresholds(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([51], warningAbove=50, criticalAbove=40)
        self.assertEqual(eo.getExitCode(), 2)

    def test_numberUnits(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([45], warningAbove=40, numberUnits="sec")
        self.assertEqual(eo.warning[0], "45sec is > 40sec")

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([45], warningAbove=40, prefixText="Test ")
        self.assertEqual(eo.warning[0], "Test 45 is > 40")

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([45], warningAbove=40, postfixText=" Units")
        self.assertEqual(eo.warning[0], "45 is > 40 Units")

    def test_bin_valuelist(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([5, 200, 52, 4000], warningAbove=100, criticalAbove=2000)
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)

    def test_emptyList(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc([], emptyStatus="OK")
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)
        ev.evalListNumberAsc([], emptyStatus="WARNING")
        self.assertEqual(eo.warning[0], "list is Empty")
        self.assertEqual(eo.getExitCode(), 1)
    
    def test_string(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberAsc(50, warningAbove=25, criticalAbove=100)
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.warning[0], "50 is > 25")
        self.assertEqual(eo.getExitCode(), 1)


class TestNagEval_evalNumberDesc(TestCase):

    def test_low(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(20, warningBelow=50, criticalBelow=40)
        self.assertEqual(eo.getExitCode(), 2)
    
    def test_mid(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(45, warningBelow=50, criticalBelow=40)
        self.assertEqual(eo.getExitCode(), 1)

    def test_high(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(55, warningBelow=50, criticalBelow=40)
        self.assertEqual(eo.getExitCode(), 0)

    def test_negative_low(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(-20, warningBelow=-5, criticalBelow=-10)
        self.assertEqual(eo.getExitCode(), 2)
    
    def test_negative_mid(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(-8, warningBelow=-5, criticalBelow=-10)
        self.assertEqual(eo.getExitCode(), 1)
    
    def test_negative_high(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(-3, warningBelow=-5, criticalBelow=-10)
        self.assertEqual(eo.getExitCode(), 0)

    def test_swappedTresholds(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(21, warningBelow=50, criticalBelow=40)
        self.assertEqual(eo.getExitCode(), 2)

    def test_numberUnits(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(45, warningBelow=50, numberUnits="sec")
        self.assertEqual(eo.warning[0], "45sec is < 50sec")

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(45, warningBelow=50, prefixText="Test ")
        self.assertEqual(eo.warning[0], "Test 45 is < 50")

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalNumberDesc(45, warningBelow=50, postfixText=" Units")
        self.assertEqual(eo.warning[0], "45 is < 50 Units")


class TestNagEval_evalListNumberDesc(TestCase):

    def test_low(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([20], warningBelow=50, criticalBelow=40)
        self.assertEqual(eo.getExitCode(), 2)
    
    def test_mid(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([45], warningBelow=50, criticalBelow=40)
        self.assertEqual(eo.getExitCode(), 1)

    def test_high(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([55], warningBelow=50, criticalBelow=40)
        self.assertEqual(eo.getExitCode(), 0)

    def test_negative_low(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([-20], warningBelow=-5, criticalBelow=-10)
        self.assertEqual(eo.getExitCode(), 2)
    
    def test_negative_mid(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([-8], warningBelow=-5, criticalBelow=-10)
        self.assertEqual(eo.getExitCode(), 1)
    
    def test_negative_high(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([-3], warningBelow=-5, criticalBelow=-10)
        self.assertEqual(eo.getExitCode(), 0)

    def test_swappedTresholds(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([21], warningBelow=50, criticalBelow=40)
        self.assertEqual(eo.getExitCode(), 2)

    def test_numberUnits(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([45], warningBelow=50, numberUnits="sec")
        self.assertEqual(eo.warning[0], "45sec is < 50sec")

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([45], warningBelow=50, prefixText="Test ")
        self.assertEqual(eo.warning[0], "Test 45 is < 50")

    def test_prefixText(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([45], warningBelow=50, postfixText=" Units")
        self.assertEqual(eo.warning[0], "45 is < 50 Units")

    def test_bin_valuelist(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([5, 200, 52, 4000], warningBelow=2000, criticalBelow=100)
        self.assertEqual(len(eo.critical), 2)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)

    def test_emptyList(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc([], emptyStatus="OK")
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)
        ev.evalListNumberDesc([], emptyStatus="WARNING")
        self.assertEqual(eo.warning[0], "list is Empty")
        self.assertEqual(eo.getExitCode(), 1)
    
    def test_string(self):
        eo = NagErrors()
        ev = NagEval(eo)
        ev.evalListNumberDesc(55, warningBelow=100, criticalBelow=50)
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.warning[0], "55 is < 100")
        self.assertEqual(eo.getExitCode(), 1)



from unittest import TestCase

from NagiosCheckHelper import NagErrors

class TestNagErrors(TestCase):

    def test_addCritical(self):
        eo = NagErrors()
        eo.addCritical('Critical Error')
        self.assertEqual(len(eo.critical), 1)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 2)

    def test_addWarning(self):
        eo = NagErrors()
        eo.addWarning('Warning Error')
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 1)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 1)
    
    def test_addUnknown(self):
        eo = NagErrors()
        eo.addUnknown('Unknown Error')
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 1)
        self.assertEqual(eo.getExitCode(), 3)
    
    def test_okStatus(self):
        eo = NagErrors()
        self.assertEqual(len(eo.critical), 0)
        self.assertEqual(len(eo.warning), 0)
        self.assertEqual(len(eo.unknown), 0)
        self.assertEqual(eo.getExitCode(), 0)

    def test_formatStatus(self):
        eo = NagErrors()
        self.assertEqual(eo.formatStatus('WARNING', ['String1']), "WARNING String1\r\n")
        self.assertEqual(eo.formatStatus('WARNING', ['String1', 'String2']), "WARNING:\r\n    String1\r\n    String2\r\n")
